@@ -2,10 +2,8 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 #include "../exim.h"
 #include "rf_functions.h"
@@ -42,22 +40,20 @@ ugid->initgroups = rblock->initgroups;
 /* If there is no fixed uid set, see if there's a dynamic one that can
 be expanded and possibly looked up. */
 
-if (!ugid->uid_set && rblock->expand_uid)
+if (!ugid->uid_set && rblock->expand_uid != NULL)
   {
-  if (!route_find_expanded_user(rblock->expand_uid, rblock->drinst.name,
-			      US"router", &upw, &ugid->uid, &addr->message))
-    return FALSE;
-  ugid->uid_set = TRUE;
+  if (route_find_expanded_user(rblock->expand_uid, rblock->name, US"router",
+    &upw, &(ugid->uid), &(addr->message))) ugid->uid_set = TRUE;
+  else return FALSE;
   }
 
 /* Likewise for the gid */
 
-if (!ugid->gid_set && rblock->expand_gid)
+if (!ugid->gid_set && rblock->expand_gid != NULL)
   {
-  if (!route_find_expanded_group(rblock->expand_gid, rblock->drinst.name,
-			      US"router", &ugid->gid, &addr->message))
-    return FALSE;
-  ugid->gid_set = TRUE;
+  if (route_find_expanded_group(rblock->expand_gid, rblock->name, US"router",
+    &(ugid->gid), &(addr->message))) ugid->gid_set = TRUE;
+  else return FALSE;
   }
 
 /* If a uid is set, then a gid must also be available; use one from the passwd
@@ -65,7 +61,7 @@ lookup if it happened. */
 
 if (ugid->uid_set && !ugid->gid_set)
   {
-  if (upw)
+  if (upw != NULL)
     {
     ugid->gid = upw->pw_gid;
     ugid->gid_set = TRUE;
@@ -73,7 +69,7 @@ if (ugid->uid_set && !ugid->gid_set)
   else
     {
     addr->message = string_sprintf("user set without group for %s router",
-      rblock->drinst.name);
+      rblock->name);
     return FALSE;
     }
   }
